#############################################################################
#############################################################################
#
# latex.tcl, version 3.1:  macros, menu, and key bindings for LaTeX users
#
#############################################################################
#
# version 1.1 and 1.2 (11/10/92) by Richard T. Austin <austin@eecs.umich.edu>
# version 2.0--3.0 and 3.1 (1/96) by Tom Scavo <trscavo@syr.edu>
#
# If you make improvements to this file, please share them!
#
#############################################################################
#############################################################################

#############################################################################
#
# Autoload procedures
#
#############################################################################

proc dummyTeX {} {}
proc latexMenu {} {}

#############################################################################
#
# Flags and variables
#
#############################################################################

set true 1
set false 0

# Flags ---
#   syntax:  newModeVar <mode> <name> <initial_value> 1
newModeVar TeX deleteEnvironmentNoisily $true 1
newModeVar TeX deleteObjectNoisily $true 1
# newModeVar TeX expertMode $false 1
newModeVar TeX promptNoisily $true 1
newModeVar TeX runTeXInBackground $false 1
newModeVar TeX searchNoisily $true 1
newModeVar TeX smartDots $true 1
newModeVar TeX smartQuotes $true 1
newModeVar TeX smartScripts $true 1
newModeVar TeX useBoxMacro $true 1
# newModeVar TeX useBracketsInDisplaymath $false 1
newModeVar TeX useDollarSigns $false 1
newModeVar TeX useStatusBar $true 1
newModeVar TeX wordWrap $true 1

# Variables ---
#   syntax:  newModeVar <mode> <name> <initial_value> 0
# newModeVar TeX TeXInputs {} 0
newModeVar TeX citeCommands {cite nocite} 0
newModeVar TeX fillColumn {70} 0
# Bug---will not select arguments with embedded braces:
newModeVar TeX funcExpr {\\(sub)*section(\[.*\]|\*)?({[^{}]*})?} 0
newModeVar TeX funcExprAlt {\\section(\[.*\]|\*)?({[^{}]*})?} 0 
# newModeVar TeX funcExpr {\\((sub)*section|chapter)(\[.*\]|\*)?({[^{}]*})?} 0
# newModeVar TeX funcExprAlt {\\(section|chapter)(\[.*\]|\*)?({[^{}]*})?} 0 
newModeVar TeX leftFillColumn {0} 0
newModeVar TeX prefixString {% } 0
newModeVar TeX refCommands {ref pageref} 0
# Will not find  \<space> :
newModeVar TeX wordBreak {([a-zA-Z-0-9]+|\\(\\\*?|[^A-Za-z\t\r* \]|[A-Za-z]+\*?))} 0
newModeVar TeX wordBreakPreface {([^a-zA-Z-0-9\]|.\\)} 0
newModeVar TeX wrapBreak {([a-zA-Z-0-9]+|\\(\\\*?|[^A-Za-z\t\r* \]|[A-Za-z]+\*?))} 0
newModeVar TeX wrapBreakPreface {([^a-zA-Z-0-9\]|.\\)} 0

# The proc 'shadowUseBoxMacro' is executed automatically whenever 
# the flag 'useBoxMacro' is changed from the "check" pop-up menu.
trace variable TeXmodeVars(useBoxMacro) w shadowUseBoxMacro
proc shadowUseBoxMacro {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars
	if {$TeXmodeVars(useBoxMacro)} then {
		# undim 'boxMacroName':
		if {[info exists TeXmodeVars((boxMacroName)]} then {
			newModeVar TeX boxMacroName $TeXmodeVars((boxMacroName) 0
			unset TeXmodeVars((boxMacroName)
		} else {
			newModeVar TeX boxMacroName {includegraphics} 0
		}
	} else {
		# dim 'boxMacroName':
		if {[info exists TeXmodeVars(boxMacroName)]} then {
			newModeVar TeX {(boxMacroName} $TeXmodeVars(boxMacroName) 0
			unset TeXmodeVars(boxMacroName)
		} else {
			newModeVar TeX {(boxMacroName} {includegraphics} 0
		}
	}
}
shadowUseBoxMacro

#############################################################################
#
# Colorization
#
#############################################################################

newModeVar TeX keywordColor blue 0
newModeVar TeX sectionColor magenta 0
newModeVar TeX commentColor red 0
newModeVar TeX bracesColor green 0


# The proc 'shadowCiteRefCommands' is executed whenever the variables 
# TeXmodeVars(citeCommands) or TeXmodeVars(refCommands) are changed.
trace variable TeXmodeVars(citeCommands) w shadowCiteRefCommands
trace variable TeXmodeVars(refCommands) w shadowCiteRefCommands
proc shadowCiteRefCommands {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars
	set LaTeXClickWords {}
	# insure that 'citeCommands' contains "cite" and "nocite":
	if {[lsearch -exact $TeXmodeVars(citeCommands) cite] == -1} then {
		lappend TeXmodeVars(citeCommands) cite
	}
	if {[lsearch -exact $TeXmodeVars(citeCommands) nocite] == -1} then {
		lappend TeXmodeVars(citeCommands) nocite
	}
	foreach word $TeXmodeVars(citeCommands) { 
		append LaTeXClickWords " \\\\$word" 
	}
	# insure that 'refCommands' contains "ref" and "pageref":
	if {[lsearch -exact $TeXmodeVars(refCommands) ref] == -1} then {
		lappend TeXmodeVars(refCommands) ref
	}
	if {[lsearch -exact $TeXmodeVars(refCommands) pageref] == -1} then {
		lappend TeXmodeVars(refCommands) pageref
	}
	foreach word $TeXmodeVars(refCommands) { 
		append LaTeXClickWords " \\\\$word" 
	}
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $TeXmodeVars(keywordColor) TeX $LaTeXClickWords
}

# The proc 'shadowBoxMacroName' is executed whenever the variable 
# TeXmodeVars(boxMacroName) is changed.
trace variable TeXmodeVars(boxMacroName) w shadowBoxMacroName
proc shadowBoxMacroName {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars
	set LaTeXClickWords {}
	if {[info exists TeXmodeVars(boxMacroName)]} then {
		foreach word $TeXmodeVars(boxMacroName) { 
			append LaTeXClickWords " \\\\$word" 
		}
	} else {
		foreach word $TeXmodeVars((boxMacroName) { 
			append LaTeXClickWords " \\\\$word" 
		}
	}
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $TeXmodeVars(keywordColor) TeX $LaTeXClickWords
}

proc colorLaTeXClickCommands {} {
	global TeXmodeVars
	set LaTeXClickWords { \\documentclass \\usepackage }
	append LaTeXClickWords { \\input \\include }
	append LaTeXClickWords { \\bibliography \\bibliographystyle }
	append LaTeXClickWords { \\LoadClass \\RequirePackage }
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $TeXmodeVars(keywordColor) TeX $LaTeXClickWords
	shadowCiteRefCommands
	shadowBoxMacroName
}

proc colorLaTeXCommands {} {
	global TeXmodeVars 
	set LaTeXKeyWords {}
	# Color comments (red), braces and dollar signs (green), and 
	# all LaTeX commands ($TeXmodeVars(keywordColor)):
	regModeKeywords -i "\}" -i "\{" -i "\$" -I $TeXmodeVars(bracesColor) \
	      -e {%} -m {\\} -c $TeXmodeVars(commentColor) -k $TeXmodeVars(keywordColor) TeX $LaTeXKeyWords
	# Color sectioning commands (magenta):
	regModeKeywords -a -k $TeXmodeVars(sectionColor) TeX \
		{ \\part \\chapter \\section \\subsection \\subsubsection \\paragraph }
}

colorLaTeXCommands
colorLaTeXClickCommands

#############################################################################
#
# LaTeX core routines
#
#############################################################################

set isLaTeXLoaded 1
if { [catch {source "$HOME:Tcl:SystemCode:latexEngine.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexEngine.tcl failed"
}

#############################################################################
#
# LaTeX macros
#
#############################################################################

if { [catch {source "$HOME:Tcl:SystemCode:latexMacros.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexMacros.tcl failed"
}

#############################################################################
#
# LaTeX menu definition
#
#############################################################################

if { [catch {source "$HOME:Tcl:SystemCode:latexMenu.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexMenu.tcl failed"
} else {
	buildLaTeXMenu
}

#############################################################################
#
# LaTeX key bindings
#
#############################################################################

if { [catch {source "$HOME:Tcl:SystemCode:latexKeys.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexKeys.tcl failed"
} else {
	bindLaTeXKeys
}

#############################################################################
#
# Smart quotes and dots
#
#############################################################################

if { [catch {source "$HOME:Tcl:SystemCode:latexSmart.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexSmart.tcl failed"
}

#############################################################################
#
# Patches and bug fixes
#
#############################################################################

if { [catch {source "$HOME:Tcl:SystemCode:latexPatches.tcl"}] } {
	set isLaTeXLoaded 0
	beep
	alertnote "Loading of latexPatches.tcl failed"
}

#############################################################################
#############################################################################

if { $isLaTeXLoaded } then {
	message "LaTeX initialization complete"
} else {
	message "LaTeX initialization failed"
}
unset isLaTeXLoaded

